/**
 * Copyright 2010 Västra Götalandsregionen
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of version 2.1 of the GNU Lesser General Public
 *   License as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 *   Boston, MA 02111-1307  USA
 */

package se.vgregion.verticalprio.entity;

import java.io.Serializable;

import javax.persistence.Cacheable;
import javax.persistence.Column;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.MappedSuperclass;

import org.apache.commons.collections.BeanMap;

import se.vgregion.dao.domain.patterns.entity.AbstractEntity;

@SuppressWarnings("serial")
@MappedSuperclass
@Inheritance(strategy = InheritanceType.TABLE_PER_CLASS)
@Cacheable(value = true)
public abstract class AbstractKod extends AbstractEntity<Long> implements Serializable, Comparable<AbstractKod>,
        Cloneable {

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	private Long id;

	@Column(name = "kod")
	private String kod;

	@Column(name = "beskrivning", length = 1000)
	private String beskrivning;

	@Column(name = "kort_beskrivning", length = 1000)
	private String kortBeskrivning;

	@Override
	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getKod() {
		return kod;
	}

	public void setKod(String kod) {
		this.kod = kod;
	}

	public String getBeskrivning() {
		return beskrivning;
	}

	public void setBeskrivning(String beskrivning) {
		this.beskrivning = beskrivning;
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public String toString() {
		return getLabel();
	}

	public String getLabel() {
		if (kortBeskrivning != null) {
			return kortBeskrivning;
		}
		return (nullAsBlank(kod) + " " + nullAsBlank(beskrivning)).trim();
	}

	private String nullAsBlank(String s) {
		if (s == null) {
			return "";
		}
		return s;
	}

	public void setKortBeskrivning(String kortBeskrivning) {
		this.kortBeskrivning = kortBeskrivning;
	}

	public String getKortBeskrivning() {
		return kortBeskrivning;
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public int compareTo(AbstractKod o) {
		int other = 0, self = 0;
		if (kod != null) {
			self = hashCode();
		}
		if (o != null && o.kod != null) {
			other = o.kod.hashCode();
		}
		return self - other;
	}

	public String getKodPlusBeskrivning() {
		return (format(getKod()) + " " + format(getBeskrivning())).trim();
	}

	private String format(String s) {
		if (s == null) {
			return "";
		}
		return s.trim();
	}

	@Override
	public AbstractKod clone() {
		try {
			AbstractKod result = getClass().newInstance();
			BeanMap thisMap = new BeanMap(this);
			BeanMap resultMap = new BeanMap(result);
			resultMap.putAllWriteable(thisMap);
			result.setId(getId());
			return result;
		} catch (InstantiationException e) {
			throw new RuntimeException(e);
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		}

	}

}
