/**
 * Copyright 2010 Västra Götalandsregionen
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of version 2.1 of the GNU Lesser General Public
 *   License as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 *   Boston, MA 02111-1307  USA
 */

package se.vgregion.verticalprio.repository;

import java.util.List;

import se.vgregion.dao.domain.patterns.entity.AbstractEntity;
import se.vgregion.dao.domain.patterns.repository.Repository;

public interface GenerisktFinderRepository<T extends AbstractEntity<Long>> extends Repository<T, Long> {

    /**
     * Uses the values of a bean to find matching values.
     * 
     * If a property of the bean is not transient (either marked with the 'transient' modifier or with the
     * annotation @Transient) then that value is used in a equals or like boolean expression to find results. If
     * the value is a String then an extra check is made to look for the '*' sign - if present it is removed in
     * favor for '%' and the operation becomes 'likes' instad of '='.
     * 
     * Null-values in the properties of the bean is ignored. They do no impact on the search expression.
     * 
     * @param bean
     * @return
     */
    List<T> findByExample(T bean, Integer maxResult);

    /**
     * Getter for strings that should be the name of the properties to use when sorting the result.
     * 
     * @return the names.
     */
    List<String> getSortOrder();

    void setExtraWhere(String extraWhere);

    String getExtraWhere();

    public List<T> query(String qt, Integer maxResult, Object... values);

}